/*******************************************************************
*
*  DESCRIPTION: Atomic Model WashCycle
*
*  AUTHOR: Ibrahim Hamdy, Akram Saleh, Mohamad Najm
*
*  DATE: 27/11/2004
*
*******************************************************************/

#include "cloth.h"  // base header
#include "message.h"       // InternalMessage ....

#include "mainsimu.h"      // class MainSimulator


/*******************************************************************
* Function Name: cloth
* Description: constructor
********************************************************************/
cloth::cloth( const string &name )
: Atomic( name )
, Cin( addInputPort( "Cin" ) )
, Cout( addOutputPort( "Cout" ) )
, preparationTime( 0, 0, 5, 0 )
, time_zero( 0, 0, 1, 0)
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}
/*******************************************************************
* Function Name: initFunction
* Description: Resetea la Value
* Precondition: El tiempo del proximo event interno es Infinito
********************************************************************/
Model &cloth::initFunction()
{
	Value = 0;
	//busyvalue = 0;
	return *this ;
}
/*******************************************************************
* Function Name: externalFunction

********************************************************************/
Model &cloth::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
	Value = msg.value();
	if(Value == 1) {
		cout<<"NO clothing -- AT TIME: "<< msg.time()<<"\n" <<endl;
		holdIn(active,time_zero);
	} else {
		cout<<" NOW clothing -- AT TIME " << msg.time()<<"\n" <<endl;
		holdIn(active,preparationTime);
	}
		
   }

	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &cloth::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &cloth::outputFunction( const InternalMessage &msg )
{
    if(Value==1)
        cout<<"CAR SENT WITHOUT CLOTHING at TIME: "<< msg.time()<<"\n"<<endl;
    if(Value==2||Value==3)
    cout<< "Finished CLOTHING, NEXT AIRDRYING  -- AT TIME:"<< msg.time()<<endl;
    sendOutput( msg.time(), Cout, Value );
    cout<<"Now SENDING THE CAR FROM CLOTH -->DRY\n"<<endl;
   
	return *this;
}



