/*******************************************************************
*
*  DESCRIPTION: Atomic Model WashCycle
*
*  AUTHOR: Ibrahim Hamdy, Akram Saleh, Mohamad Najm
*
*  DATE: 27/11/2004
*
*******************************************************************/

#include "drycycle.h"  // base header
#include "message.h"       // InternalMessage ....

#include "mainsimu.h"      // class MainSimulator


/*******************************************************************
* Function Name: drycycle
* Description: constructor
********************************************************************/
drycycle::drycycle( const string &name )
: Atomic( name )
, Din( addInputPort( "Din" ) )
, Dout( addOutputPort( "Dout" ) )
, Ddone( addOutputPort( "Ddone" ) )
, preparationTime( 0, 0, 6, 0 )
, time_zero( 0, 0, 1, 0)
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}
/*******************************************************************
* Function Name: initFunction
* Description: Resetea la Value
* Precondition: El tiempo del proximo event interno es Infinito
********************************************************************/
Model &drycycle::initFunction()
{
	Value = 0;
	busy = 0;
	return *this ;
}
/*******************************************************************
* Function Name: externalFunction

********************************************************************/
Model &drycycle::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
	Value = msg.value();
	if(Value == 1) {
		cout<<"NO DRYING -- AT TIME: "<< msg.time()<<"\n" <<endl;
		holdIn(active,time_zero);
	} else {
		cout<<" NOW DRYING -- AT TIME: " << msg.time()<<"\n" <<endl;
		holdIn(active,preparationTime);
	}
		
   }

	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &drycycle::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &drycycle::outputFunction( const InternalMessage &msg )
{
    if(Value==1){
        cout<<"NO DRYING, SENDING THE CAR OUT AT TIME: "<< msg.time()<<"\n"<<endl;
    }
    if(Value==2||Value==3){
    	cout<< "Finished DRYING -- AT TIME: "<< msg.time()<<"\n"<<endl;
    }
    sendOutput( msg.time(), Dout, 999 );
    cout<<"SEND THE CAR OUT OF THE SYSTEM with a VALUE OF\n"<<endl;
    sendOutput( msg.time(), Ddone, busy);

	cout<<"TELLING THE QUEUE TO SEND THE NEXT CAR\n"<<endl;
	return *this;
}



