/*******************************************************************
*
*  DESCRIPTION: Atomic Model Queue
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*
*******************************************************************/

/** include files **/
#include "myqueue.h"      // class Queue
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: Queue
* Description: 
********************************************************************/
MYQueue::MYQueue( const string &name )
: Atomic( name )
, in( addInputPort( "in" ) )
, done( addInputPort( "done" ) )
, out( addOutputPort( "out" ) )
, preparationTime( 0, 0, 2, 0 )
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}

/*******************************************************************
* Function Name: initFunction
* Description: Resetea la lista
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &MYQueue::initFunction()
{
	cout<<"THIS IS THE QUEUE:\n"<<endl;
	elements.erase( elements.begin(), elements.end() ) ;
	elements.size() == 0;	// element list size is 0
	busy = 0;				// washsystem is not busy.
	return *this ;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &MYQueue::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == in )
	{
		cout<<"New car is added to the queue at time: "<<msg.time()<<"\n"<<endl;
		elements.push_back( msg.value() ) ;
	}

	if( msg.port() == done )
	{
	cout<<" A SIGNAL HAS BEEN SENT TO RESUME OR STOP SENDING FROM QUEUE\n"<<endl;
		busy = msg.value();
	}
	if(busy == 0)	// SEND A CAR IF BUSY = 0
		holdIn(active,preparationTime);
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &MYQueue::internalFunction( const InternalMessage & )
{
	
	passivate();
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &MYQueue::outputFunction( const InternalMessage &msg )
{
	if(elements.size()!=0){
		cout<<"Sending a Car to PCODE at time: "<<msg.time()<<"THEN TO THE CAR SYSTEM\n"<<endl;
		sendOutput( msg.time(), out, elements.front() ) ;
		elements.pop_front();	// delete first element, after we have sent it.
	busy=1;
	}
	return *this ;
}
