/*******************************************************************
*
*  DESCRIPTION: Atomic Model Queue
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*
*******************************************************************/

#ifndef __MYQUEUE_H
#define __MYQUEUE_H

#include <list>
#include "atomic.h"     // class Atomic

class MYQueue : public Atomic
{
public:
	MYQueue( const string &name = "MYQueue" );					//Default constructor

	virtual string className() const ;
protected:
	Model &initFunction();
	Model &externalFunction( const ExternalMessage & );
	Model &internalFunction( const InternalMessage & );
	Model &outputFunction( const InternalMessage & );

private:
	const Port &in;
	const Port &done;				// an input port to change the value of busy
	Port &out;
	Time preparationTime;
	typedef list<Value> ElementList ;
	ElementList elements ;
	int busy;					// 1 = busy, 0 = not busy.
									// busy refers to the washsystem, if it is busy we can't send.
									// otherwise we can send a car to the system. initially the 
									// system is not busy.		
	Time timeLeft;

};	// class Queue

// ** inline ** // 
inline
string MYQueue::className() const
{
	return "MYQueue" ;
}

#endif   //__MYQUEUE_H
