/*******************************************************************
*
*  DESCRIPTION: Atomic Model WashCycle
*
*  AUTHOR: Ibrahim Hamdy, Akram Saleh, Mohamad Najm
*
*  DATE: 27/11/2004
*
*******************************************************************/

#include "shampoo.h"  // base header
#include "message.h"       // InternalMessage ....

#include "mainsimu.h"      // class MainSimulator


/*******************************************************************
* Function Name: washcycle
* Description: constructor
********************************************************************/
shampoo::shampoo( const string &name )
: Atomic( name )
, Sin( addInputPort( "Sin" ) )
, Sout( addOutputPort( "Sout" ) )
, preparationTime( 0, 0, 2, 0 )
, time_zero( 0, 0, 1, 0)
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}
/*******************************************************************
* Function Name: initFunction
* Description: Resetea la Value
* Precondition: El tiempo del proximo event interno es Infinito
********************************************************************/
Model &shampoo::initFunction()
{
	Value = 0;
	return *this ;
}
/*******************************************************************
* Function Name: externalFunction

********************************************************************/
Model &shampoo::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
	Value = msg.value();
	if(Value == 3) {
		holdIn(active,time_zero);
		cout<<"NO SHAMPOOING: at time"<< msg.time()<<"\n"<<endl;
	} else {
		holdIn(active,preparationTime);
			cout<<" NOW Shampooing: at Time " << msg.time()<<"\n" <<endl;
	}
		
   }

	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &shampoo::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &shampoo::outputFunction( const InternalMessage &msg )
{
    if(Value==3)
        cout<<"IN OUTPUT NO SHAMPOOING"<< msg.time()<<"\n"<<endl;
    if((Value==1||Value==2) && (Value!=3))
    cout<< "Finished Shampooing at time	"<< msg.time()<<"\n"<<endl;    
    cout<<"Value sent through Shampoo:	"<<Value<<"\n"<<endl;
    sendOutput( msg.time(), Sout, Value );
	cout<<"This is to check the SOUT\n"<<endl;
	//sendOutput( msg.time(), busy, busyValue);
	//cout<<"THIS to chech busy"<<endl;
	return *this;
}