/*******************************************************************
*
*  DESCRIPTION: Atomic Model WashCycle
*
*  AUTHOR: Ibrahim Hamdy, Akram Saleh, Mohamad Najm
*
*  DATE: 27/11/2004
*
*******************************************************************/

#include "washcycle.h"  // base header
#include "message.h"       // InternalMessage ....

#include "mainsimu.h"      // class MainSimulator


/*******************************************************************
* Function Name: washcycle
* Description: constructor
********************************************************************/
washcycle::washcycle( const string &name )
: Atomic( name )
, Washin( addInputPort( "Washin" ) )
, Washout( addOutputPort( "Washout" ) )
, preparationTime( 0, 0, 3, 0 )
, time_zero( 0, 0, 1, 0)
{
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		preparationTime = time ;
}
/*******************************************************************
* Function Name: initFunction
* Description: Resetea la Value
* Precondition: El tiempo del proximo event interno es Infinito
********************************************************************/
Model &washcycle::initFunction()
{
	Value = 0;
	//busyValue = 1;
	return *this ;
}
/*******************************************************************
* Function Name: externalFunction

********************************************************************/
Model &washcycle::externalFunction( const ExternalMessage &msg )
{
  if ( this->state() == passive)
  {
	Value = msg.value();
	if(Value == 3) {
		holdIn(active,time_zero);
		cout<<"NO WASHING: at time"<< msg.time()<<"\n" <<endl;
	} else {
		cout<<" NOW washing: at Time" << msg.time()<<"\n" <<endl;
		holdIn(active,preparationTime);
		}
		
   }

	return *this ;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &washcycle::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &washcycle::outputFunction( const InternalMessage &msg )
{
    if(Value==3)
        cout<<"THE CAR HAS NOT BEEN WASHED at time"<< msg.time()<<"\n"<<endl;
    if((Value==1||Value==2) && (Value!=3))
    cout<< "Finished Washing at time"<< msg.time()<<"\n"<<endl;    
    cout<<"Value sent through Washcycle is : "<<Value<<"\n"<<endl;
    sendOutput( msg.time(), Washout, Value );
	cout<<"CAR SENT FROM WASH --> RINSE\n"<<endl;
	//sendOutput( msg.time(), busy, busyValue);
	//cout<<"THIS to chech busy"<<endl;
	return *this;
}